/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.

  FILE:		UdmInt.h
  SYNOPSIS:	Udm Interface to EiC interpretter engine
  CREATED:	08/10/02
  AUTHOR:	Endre Magyari(endre.magyari@vanderbilt.edu)

  CHANGELOG:

  08/10/02	- Added Support for EiC C interpreter. See details 
				at EiC_Interpreter_iface_section



*/

#ifndef UDMINT_H
#define UDMINT_H



/* EiC_Interpreter_iface_section
 *
 *		EiC interpreter interface
 *	supported features: 
 *	
 *		08/10/02	- endre
 *
 *			- UdmPseudoObject struct
 *			- automatic global UdmPseudoObject declaration & bounding for 
 *				any Udm::Object
 *			- Set/Get Int/Real/String/Bool attributes for UdmPseudoObjects
 *				bound to real Udm::Object-s
 *			- interactive Udm EiC ANSI C console (Standard C library supported)
 *		
 */

#ifdef UDM_EIC_SUPPORT

//EIC is ANSI C - no support for bool
typedef char EIC_BOOL ;
#define EIC_FALSE 0x00
#define EIC_TRUE 0x01


//init the EiC C interpreter
void eic_init(char *eic_switches, void (*msg_handler)(char *));

//add an object o with name as a variable name as a global UdmPseudoObject
//name must be unique, or clear it first
void eic_add_object(Udm::Object o, string name);

//remove the global var name with name 'name'
void eic_rem_object(string name, bool throw_not_found = true);

//clear all global variables added by eic_add_object
void eic_clear();

//start interactive mode
void eic_ipretter();

//execute code
extern "C" void EiC_parseString(char *, ...);

#endif UDM_EIC_SUPPORT

/*
 *		Cint interpreter UDM interface 
 *	supported features: 
 *	-	initialize
 *	-	destroy
 *	-	interactive mode
 *	-	add initialized UdmPseudoObject to current context
 *	-	remove variable from global scope
 *	-	evaluate simple expressions
 *	-	evaluate loops, conditionals, declarations
 *	-	load/unload source code including function defintions(CINT API functions)
 *	-	... for the complete CInt API refer to CInt manual
 */


//init the CInt C/C++ interpreter
void cint_init(char * cint_switches, void (*msg_handler)(char *));

//destroy the interpretter
void cint_destroy();


//add an object o to the existing code block
//two objects with the same name can not be declared in the {same block}.
void cint_add_object(Udm::Object o, string name);

//starts interactive mode
void cint_ipretter();

//eval an expression and return value as int
//no declaration and conditional statement
int cint_int_calc(string code);

//eval an expression and return value as double
//no declaration and conditional statement
double cint_double_calc(string code);

//eval an expression and return value as int
//can contain,
//variable   declaration,   struct,union,enum    definition.
//expression,  loop and control statement and cannot contain
//function definition.  G__exec_tempfile() executes a state-
//ment or statement list surrounded by '{' , '}' and returns
//the last evaluated value.
int cint_int_eval(string code);

//eval an expression and return value as double
double cint_double_eval(string code);

char* cint_load_text(const char *namedmacro);

/*
 G__loadfile() API loads source code or DLL(Dynamic Link Library) at run-time.
If file extention is .sl  , .dl , .dll or .DLL , the file is linked as DLL.
Otherwise, the file is loaded as source or header file.
*/
int cint_loadfile(const char *filename);

/*
 G__unloadfile() API unloads source code or DLL(Dynamic Link Library) at 
run-time.  Cint loads/unloads source/DLL files in stack-wise way. All the
source/DLL files loaded after specified one will be unloaded at once.
*/
int cint_unloadfile(const char *filename);


#endif UDMINT_H